require_relative '../lib/cloudutil/aws.rb'

describe Cloudutil::AWS::EC2 do
  context 'object initialization' do
    it 'creates an object using primary env vars' do
      ENV.clear
      ENV['AWS_ACCESS_KEY_ID'] = 'my_primary_access_key'
      ENV['AWS_SECRET_ACCESS_KEY'] = 'my_primary_secret_key'

      aws = Cloudutil::AWS::EC2.new

      expect(aws.region).to eq 'us-east-1'
      expect(aws.access_key).to eq 'my_primary_access_key'
      expect(aws.secret_key).to eq 'my_primary_secret_key'
    end

    it 'creates an object using implicit hash args' do
      reg = 'us-west-1'
      ak  = 'my_access_key1'
      sk  = 'my_secret_key1'

      aws = Cloudutil::AWS::EC2.new(region: reg, access_key: ak, secret_key: sk)

      expect(aws.region).to eq reg
      expect(aws.access_key).to eq ak
      expect(aws.secret_key).to eq sk
    end
  end

  context 'resolving VPC subnets' do
    # Need to do this to read AWS creds from env
    my_env = ENV.to_h
    before(:each) do
      @aws = Cloudutil::AWS::EC2.new(access_key: my_env['AWS_ACCESS_KEY_ID'], secret_key: my_env['AWS_SECRET_ACCESS_KEY'])
    end

    it 'returns what we passed in, if it looks like an AWS subnet id' do
      id = @aws.resolve_subnet_id('subnet-deadbeef')
      expect(id).to eq 'subnet-deadbeef'
    end

    it 'returns a known VPC subnet object, case-sensitive', account_specific: true do
      id = @aws.resolve_subnet_id('vpc-private-azA', false)

      expect(id).to_not be_nil
      expect(id).to match(/^subnet-\h{8,}$/)
    end

    it 'returns a known VPC subnet object, case-insensitive', account_specific: true do
      id = @aws.resolve_subnet_id('VPC-Private-AZa')

      expect(id).to_not be_nil
      expect(id).to match(/^subnet-\h{8,}$/)
    end

    it 'returns nil for no case-sensitive VPC subnet match', account_specific: true do
      id = @aws.resolve_subnet_id('VPC-private-aza', false)
      expect(id).to be_nil
    end

    it 'returns nil if given a completely bogus name' do
      id = @aws.resolve_subnet_id('my_totally_bogus_subnet_name')
      expect(id).to be_nil
    end
  end

  context 'resolving security groups' do
    my_env = ENV.to_h
    before(:each) do
      @aws = Cloudutil::AWS::EC2.new(access_key: my_env['AWS_ACCESS_KEY_ID'], secret_key: my_env['AWS_SECRET_ACCESS_KEY'])
    end

    it 'returns what we pass in, if it looks like an AWS security group id' do
      id = @aws.resolve_security_group_id('sg-deadbeef')
      expect(id).to eq 'sg-deadbeef'
    end

    it 'returns a known security group by name, case-sensitive', account_specific: true do
      id = @aws.resolve_security_group_id('VPC_Private', false)

      expect(id).to_not be_nil
      expect(id).to match(/^sg-\h{8,}$/)
    end

    it 'returns a known security group by name, case-insensitive', account_specific: true do
      id = @aws.resolve_security_group_id('vpc_PRIVATE')

      expect(id).to_not be_nil
      expect(id).to match(/^sg-\h{8,}$/)
    end

    it 'returns a known security group by tag, case-sensitive', account_specific: true do
      id = @aws.resolve_security_group_id('vpc-private', false)

      expect(id).to_not be_nil
      expect(id).to match(/^sg-\h{8,}$/)
    end

    it 'returns a known security group by tag, case-insensitive', account_specific: true do
      id = @aws.resolve_security_group_id('VPC-PrIvAtE')

      expect(id).to_not be_nil
      expect(id).to match(/^sg-\h{8,}$/)
    end

    it 'returns nil for no case-sensitive security group name or tag match', account_specific: true do
      id = @aws.resolve_security_group_id('vpc_PrivatE', false)
      expect(id).to be_nil
    end

    it 'returns nil if given a completely bogus name/tag value' do
      id = @aws.resolve_subnet_id('my_totally_bogus_subnet_name')
      expect(id).to be_nil
    end
  end

  context 'resolving AMIs' do
    my_env = ENV.to_h
    before(:each) do
      @aws = Cloudutil::AWS::EC2.new(access_key: my_env['AWS_ACCESS_KEY_ID'], secret_key: my_env['AWS_SECRET_ACCESS_KEY'])
    end

    it 'returns what we pass in, if it looks like an AWS ami id' do
      id = @aws.resolve_ami_id('ami-deadbeef')
      expect(id).to eq 'ami-deadbeef'
    end

    it 'returns a known AMI by name, case-sensitive', account_specific: true do
      id = @aws.resolve_ami_id('AMIBakery-MRE-28', false)

      expect(id).to_not be_nil
      expect(id).to match(/^ami-\h{8,}$/)
    end

    it 'returns a known AMI by name, case-insensitive', account_specific: true do
      id = @aws.resolve_ami_id('amiBAKERY-mre-28')

      expect(id).to_not be_nil
      expect(id).to match(/^ami-\h{8,}$/)
    end

    it 'returns a known AMI by tag, case-sensitive', account_specific: true do
      id = @aws.resolve_ami_id('pearson_jenkins_ubuntu', false)

      expect(id).to_not be_nil
      expect(id).to match(/^ami-\h{8,}$/)
    end

    it 'returns a known AMI by tag, case-insensitive', account_specific: true do
      id = @aws.resolve_ami_id('Pearson_JENKINS_uBuntu')

      expect(id).to_not be_nil
      expect(id).to match(/^ami-\h{8,}$/)
    end

    # WARNING: This test can take over a minute to run, since it has to inspect the
    # properties of all 31k public AMIs + any private AMIs my account has access to.
    it 'returns nil for no case-sensitive AMI name or tag match', slow: true, account_specific: true do
      id = @aws.resolve_ami_id('amiBakery-mRe-28', false)
      expect(id).to be_nil
    end

    # WARNING: This test can take over a minute to run, since it has to inspect the
    # properties of all 31k public AMIs + any private AMIs my account has access to.
    it 'returns nil if given a completly bogus name/tag value', slow: true do
      id = @aws.resolve_ami_id('my_totally_bogus_ami_name')
      expect(id).to be_nil
    end
  end
end
